@@ -8,24 +8,22 @@ use warnings;
 use base 'Mojo::Base';
 
 use Carp 'croak';
-use Mojo::File;
+use Mojo::Asset::File;
 use Mojo::Headers;
 
-__PACKAGE__->attr('file', default => sub { Mojo::File->new });
+__PACKAGE__->attr(asset => sub { Mojo::Asset::File->new });
 __PACKAGE__->attr([qw/filename name/]);
-__PACKAGE__->attr('headers', default => sub { Mojo::Headers->new });
+__PACKAGE__->attr(headers => sub { Mojo::Headers->new });
 
 # B-6
 # You sunk my scrabbleship!
 # This game makes no sense.
 # Tell that to the good men who just lost their lives... SEMPER-FI!
-sub copy_to { shift->file->copy_to(@_) }
+sub move_to { shift->asset->move_to(@_) }
 
-sub length { shift->file->length }
+sub size { shift->asset->size }
 
-sub move_to { shift->file->move_to(@_) }
-
-sub slurp { shift->file->slurp }
+sub slurp { shift->asset->slurp }
 
 1;
 __END__
@@ -40,7 +38,7 @@ Mojo::Upload - Upload
 
     my $upload = Mojo::Upload->new;
     print $upload->filename;
-    $upload->copy_to('/foo/bar.txt');
+    $upload->move_to('/foo/bar/baz.txt');
 
 =head1 DESCRIPTION
 
@@ -48,36 +46,23 @@ L<Mojo::Upload> is a container for uploads.
 
 =head1 ATTRIBUTES
 
-=head2 C<file>
+L<Mojo::Upload> implements the following attributes.
 
-    my $file = $upload->file;
-    $upload  = $upload->file(Mojo::File->new);
+=head2 C<asset>
 
-Returns a L<Mojo::File> object if called without arguments.
-Returns the invocant if called with arguments.
+    my $asset = $upload->asset;
+    $upload   = $upload->asset(Mojo::Asset::File->new);
 
 =head2 C<filename>
 
     my $filename = $upload->filename;
     $upload      = $upload->filename('foo.txt');
 
-Returns a file name like C<foo.txt> if called without arguments.
-Returns the invocant if called with arguments.
-
-=head2 C<length>
-
-    my $length = $upload->length;
-
-Returns the length of the file upload in bytes.
-
 =head2 C<headers>
 
     my $headers = $upload->headers;
     $upload     = $upload->headers(Mojo::Headers->new);
 
-Returns a L<Mojo::Headers> object if called without arguments.
-Returns the invocant if called with arguments.
-
 =head2 C<name>
 
     my $name = $upload->name;
@@ -88,20 +73,16 @@ Returns the invocant if called with arguments.
 L<Mojo::Upload> inherits all methods from L<Mojo::Base> and implements the
 following new ones.
 
-=head2 C<copy_to>
-
-    $upload->copy_to('/foo/bar/baz.txt');
-
-Copies the uploaded file contents to the given path.
-
 =head2 C<move_to>
 
     $upload->move_to('/foo/bar/baz.txt');
 
-Moves the uploaded file contents to the given path.
+=head2 C<size>
+
+    my $size = $upload->size;
 
 =head2 C<slurp>
 
-    my $content = $upload->slurp;
+    my $string = $upload->slurp;
 
 =cut
